﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using OpenTK;
using OpenTK.Graphics.OpenGL;
using OpenTK.Platform.Windows;


namespace WindowsFormsApplication2
{
    public partial class Form1 : Form
    {
        double AngleX = 0;
        double AngleY = 0;
        double AngleZ = 0;


        const double Step = 0.2;   // шаг перемещения наблюдателя в лабиринте
        const double hdl = 5;        // высота стен лабиринта
        const string s = "Open GL";  // трехмерная вращающаяся надпись в лабиринте
        const double AngleDl = 5;

        // координаты положения наблюдателя в лабиринте
        PointFloat Pos;

        
        // структура для описания расположения стены
        public struct Wall 
        {
            public int L;
            public int R;
            public Color C;
        }

        // координаты углов лабиринта
        public struct PointFloat
        {
            public double X;
            public double Y;
        }
        
        PointFloat[] coners = new []{
            new PointFloat{X = -10, Y = -10},
            new PointFloat{X = -10, Y = 10},
            new PointFloat{X =  10, Y = 10},
            new PointFloat{X =  10, Y = -10},
            new PointFloat{X =  3, Y = 10},
            new PointFloat{X =  3, Y = 0}
        };

        // расположение стен в лабиринте
        Wall[] plan = new[]
        {
            new Wall{L = 0, R = 1, C = Color.Red},
            new Wall{L = 1, R = 2, C = Color.Green},
            new Wall{L = 2, R = 3, C = Color.Yellow},
            new Wall{L = 3, R = 0, C = Color.Blue},
            new Wall{L = 4, R = 5, C = Color.Aqua}
        };

  

        public Form1()
        {
            InitializeComponent();
        }

        private void glControl1_Resize(object sender, EventArgs e)
        {
            GL.Viewport(0, 0, glControl1.Width, glControl1.Height);

            GL.MatrixMode(MatrixMode.Projection);
            GL.LoadIdentity();
            GL.Frustum(-0.5, 0.5, -0.5, 0.5, 0.5, 50);
            GL.MatrixMode(MatrixMode.Modelview);
            glControl1.Invalidate();
        }

        private void glControl1_Paint(object sender, PaintEventArgs e)
        {
            GL.ClearColor(0.5f, 0.5f, 0.75f, 1.0f); // цвет фона
            // очистка буферов цвета и глубины
            GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);
            // поворот изображения
            GL.LoadIdentity();

            GL.Rotate(AngleY, 0.0, 1.0, 0.0);
            GL.Translate(Pos.X, 0, Pos.Y);

             // стены лабиринта
            for (int i = 0; i < plan.Length; ++i)
            {
                // длина стены
                double wdl = Math.Sqrt(Math.Pow(coners[plan[i].L].X - coners[plan[i].R].X, 2)
                  + Math.Pow(coners[plan[i].L].Y - coners[plan[i].R].Y, 2)) / 4;
                GL.Color4(plan[i].C);
                GL.Begin(BeginMode.Quads);
                    GL.TexCoord2(0, 0);         GL.Vertex3(coners[plan[i].L].X, -hdl, coners[plan[i].L].Y);
                    GL.TexCoord2(0, hdl / 2);   GL.Vertex3(coners[plan[i].L].X,  hdl, coners[plan[i].L].Y);
                    GL.TexCoord2(wdl, hdl / 2); GL.Vertex3(coners[plan[i].R].X,  hdl, coners[plan[i].R].Y);
                    GL.TexCoord2(wdl, 0);       GL.Vertex3(coners[plan[i].R].X, -hdl, coners[plan[i].R].Y);
                GL.End();
            }

            GL.Flush();
            GL.Finish();

            glControl1.SwapBuffers();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            GL.Enable(EnableCap.DepthTest);

            Pos.Y = -3;
        }

        private void glControl1_KeyDown(object sender, KeyEventArgs e)
        {
            switch(e.KeyCode)
            {
                case Keys.Left: AngleY += -AngleDl; break;
                case Keys.Right: AngleY -= -AngleDl; break;
                case Keys.Up:
                    Pos.X = Pos.X - Step * Math.Sin(AngleY * Math.PI / 180);
                    Pos.Y = Pos.Y + Step * Math.Cos(AngleY * Math.PI / 180);
                break;
                case Keys.Down:
                    Pos.X = Pos.X + Step * Math.Sin(AngleY * Math.PI / 180);
                    Pos.Y = Pos.Y - Step * Math.Cos(AngleY * Math.PI / 180);
                break;
            }
            if (AngleY>=360) AngleY -= 360;
            glControl1.Invalidate(); 
        }


        // изображение сферы
        void Sphere(double r, int nx, int ny)
        {
            int ix, iy;
            double x, y, z;
        
            for (iy = 0; iy < ny; ++iy)
            {
                GL.Begin(BeginMode.QuadStrip);
                for (ix=0; ix<= nx; ++ix)
                {
                    //double d = iy * Math.PI / ny;
                    //x= Math.Sin (d);
                  x= r*Math.Sin(iy*Math.PI/ny)*Math.Cos(2*ix*Math.PI/nx);

                  y = r * Math.Sin(iy * Math.PI / ny) * Math.Sin(2 * ix * Math.PI / nx);
                  z = r * Math.Cos(iy * Math.PI / ny);
                  GL.Normal3(x, y, z);//нормаль направлена от центра
                  GL.TexCoord2((double)ix / (double)nx, (double)iy / (double)ny);
                  GL.Vertex3(x, y, z);

                  x = r * Math.Sin((iy + 1) * Math.PI / ny) * Math.Cos(2 * ix * Math.PI / nx);
                  y=r * Math.Sin ((iy+1)*Math.PI/ny)*Math.Sin(2*ix*Math.PI/nx);
                  z = r * Math.Cos((iy + 1) * Math.PI / ny);
                  GL.Normal3(x, y, z);
                  GL.TexCoord2((double)ix / (double)nx, (double)(iy + 1) / (double)ny);
                  GL.Vertex3(x, y, z);
                }
                GL.End();
            }
        }

        private void glControl1_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Insert:
                    AngleX += AngleDl;
                    break;
                case Keys.Delete:
                    AngleX -= AngleDl;
                    break;
                case Keys.Home:
                    AngleY += AngleDl;
                    break;
                case Keys.End:
                    AngleY -= AngleDl;
                    break;
                case Keys.Prior:
                    AngleZ += AngleDl;
                    break;
                case Keys.Next:
                    AngleZ -= AngleDl;
                    break;


                case Keys.Left:
                    AngleY += -AngleDl;
                    break;
                case Keys.Right:
                    AngleY -= -AngleDl;
                    break;
                case Keys.Up:
                    Pos.X = Pos.X - Step * Math.Sin(AngleY * Math.PI / 180);
                    Pos.Y = Pos.Y + Step * Math.Cos(AngleY * Math.PI / 180);
                    break;
                case Keys.Down:
                    Pos.X = Pos.X + Step * Math.Sin(AngleY * Math.PI / 180);
                    Pos.Y = Pos.Y - Step * Math.Cos(AngleY * Math.PI / 180);
                    break;
 

            }

            if (AngleY >= 360)
                AngleY -= 360;

            glControl1.Invalidate(); 
        }

    }
}
