unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OpenGL, ExtCtrls, uTexture;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormResize(Sender: TObject);
  private
    DC : HDC;
    hrc: HGLRC;
  end;
var
  Form1: TForm1;

implementation

{$R *.DFM}
type
  //     
  TWall = record
    l, r: Integer;
    c: TColor;
  end;

const
  //   
  coners: array [0..5] of TPointFloat = (
        (x:-10; y:-10;),
        (x:-10; y: 10),
        (x: 10; y: 10),
        (x: 10; y:-10),
        (x: 3; y:-10),
        (x: 3; y: 0));
  //    
  plan: array [0..4] of TWall = (
    (l:0;r:1;c:clRed),(l:1;r:2;c:clGreen),
    (l:2;r:3;c:clYellow),(l:3;r:0;c:clBlue),
    (l:4;r:5;c: clAqua));

  AngleDl=5;    //     
  Step = 0.2;   //     
  hdl=5;        //   
  s='Open GL';  //     

var
  //      
  texBrick, texEarth: TGLTexture;
  //    (  ),
  //         
  AngleY: integer;
  //     
  Pos: TPointFloat;
  //      , 
  //    
  RY: integer;


procedure TForm1.FormPaint(Sender: TObject);
var
  i: integer;
  wdl: GLfloat;
begin
 glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);      //   
 glLoadIdentity;

 //  
 glRotatef(AngleY, 0.0, 1.0, 0.0);
 glTranslatef(Pos.X, 0, Pos.Y);


 //  
 for i:=0 to High(plan) do begin
   //  
   wdl:=sqrt(sqr(coners[plan[i].l].x-coners[plan[i].r].x)
     + sqr(coners[plan[i].l].y-coners[plan[i].r].y))/4;
   glBegin(GL_QUADS);
     glColor(GetRValue(plan[i].c), GetGValue(plan[i].c), GetBValue(plan[i].c));
     glTexCoord2f(0, 0);      glVertex3f(coners[plan[i].l].x, -hdl, coners[plan[i].l].y);
     glTexCoord2f(0, hdl/2);  glVertex3f(coners[plan[i].l].x, hdl, coners[plan[i].l].y);
     glTexCoord2f(wdl,hdl/2); glVertex3f(coners[plan[i].r].x, hdl, coners[plan[i].r].y);
     glTexCoord2f(wdl, 0);    glVertex3f(coners[plan[i].r].x, -hdl, coners[plan[i].r].y);
   glEnd;
 end;

 glFlush;
 glFinish;

 SwapBuffers(DC);
end;

procedure SetDCPixelFormat (hdc : HDC);
var
  pfd : TPixelFormatDescriptor;
  nPixelFormat : Integer;
begin
  FillChar (pfd, SizeOf (pfd), 0);
  pfd.dwFlags  := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER;
  nPixelFormat := ChoosePixelFormat (hdc, @pfd);
  SetPixelFormat (hdc, nPixelFormat, @pfd);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  DC := GetDC (Handle);
  SetDCPixelFormat(DC);
  hrc := wglCreateContext(DC);
  wglMakeCurrent(DC, hrc);
  glClearColor (0.5, 0.5, 0.75, 1.0); //  
  glEnable(GL_DEPTH_TEST);

  //    
  wglUseFontOutLines(Form1.Canvas.Handle, 0, 255, 1000, 0, 0.5, WGL_FONT_POLYGONS, nil);
  //   
  LoadTextureFromBMP('BRICK.BMP', texBrick);
  LoadTextureFromBMP('earth256x128.bmp', texEarth);
  //   
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
  //
  Pos.Y:=-3;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 wglMakeCurrent(0, 0);
 wglDeleteContext(hrc);
 ReleaseDC (Handle, DC);
 DeleteDC (DC);
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 If Key = VK_ESCAPE then Close;
 case Key of
  VK_LEFT:   inc(AngleY, -AngleDl);
  VK_RIGHT:  dec(AngleY, -AngleDl);
  VK_UP:
    begin
      Pos.X:=Pos.X - Step*Sin(AngleY*pi/180);
      Pos.Y:=Pos.Y + Step*Cos(AngleY*pi/180);
    end;
  VK_DOWN:
    begin
      Pos.X:=Pos.X + Step*Sin(AngleY*pi/180);
      Pos.Y:=Pos.Y - Step*Cos(AngleY*pi/180);
    end;
 end;
 if AngleY>=360 then dec (AngleY, 360);
 InvalidateRect(Handle, nil, False);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
 glViewport(0, 0, ClientWidth, ClientHeight);
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity;
 glFrustum(-0.5, 0.5, -0.5, 0.5, 0.5, 50);
 glMatrixMode(GL_MODELVIEW);
 InvalidateRect(Handle, nil, False);
end;

end.

